
package io.platform.sys.web.remote;

import com.alibaba.dubbo.config.annotation.Reference;
import com.baomidou.mybatisplus.plugins.Page;
import com.google.code.kaptcha.Producer;
import io.platform.base.common.exception.RRException;
import io.platform.base.common.utils.DateUtils;
import io.platform.base.common.utils.ExecuteResult;
import io.platform.base.common.utils.Query;
import io.platform.sys.dto.SysCaptchaDto;
import io.platform.sys.service.client.api.SysCaptchaServiceClient;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.awt.image.BufferedImage;
import java.util.Date;
import java.util.List;
import java.util.Map;


/**
 * Created by sevelli on 2017/8/31.
 */
@Service("sysCaptchaServiceRemote")
public class SysCaptchaServiceRemote {

    @Reference(check = false)
    private SysCaptchaServiceClient sysCaptchaServiceClient;

    @Autowired
    private Producer producer;
    /**
     * 根据参数获取列表  无分页
     */
    public List<SysCaptchaDto> getList(Map<String, Object> params){
        return sysCaptchaServiceClient.queryList(params);
    }
    /**
     * 根据参数进行 分页查询
     */
    public Page<SysCaptchaDto> getPage(Page<SysCaptchaDto> page, SysCaptchaDto sysCaptchaDto){
        return sysCaptchaServiceClient.getPage(page,sysCaptchaDto);
    }
    /**
     * 根据主键 获取单条数据明细
     */
    public SysCaptchaDto getOne(String id){
        SysCaptchaDto sysCaptchaDto = sysCaptchaServiceClient.queryObject(id);
        return sysCaptchaDto;
    }
    /**
     * 保存 数据
     */
    public ExecuteResult<String> save(SysCaptchaDto sysCaptchaDto){

       return  sysCaptchaServiceClient.save(sysCaptchaDto);
    }
    /**
     * 修改单条数据
     */
    public ExecuteResult<String> update(SysCaptchaDto sysCaptchaDto){

        return sysCaptchaServiceClient.update(sysCaptchaDto);
    }
    /**
     * 批量删除
     */
    public ExecuteResult<String> deleteBatch(List<String> ids){
        return  sysCaptchaServiceClient.deleteBatch(ids);
    }


    public BufferedImage getCaptcha(String uuid) {
        if(StringUtils.isBlank(uuid)){
            throw new RRException("uuid不能为空");
        }
        //生成文字验证码
        String code = producer.createText();

        SysCaptchaDto captchaEntity = new SysCaptchaDto();
        captchaEntity.setUuid(uuid);
        captchaEntity.setCode(code);
        //5分钟后过期
        captchaEntity.setExpireTime(DateUtils.addDateMinutes(new Date(), 5));
        sysCaptchaServiceClient.save(captchaEntity);

        return producer.createImage(code);
    }

    public boolean validate(String uuid, String captcha) {
        SysCaptchaDto captchaEntity = sysCaptchaServiceClient.queryObject(uuid);
        if(captchaEntity == null){
            return false;
        }

        //删除验证码
        sysCaptchaServiceClient.delete(uuid);

        if(captchaEntity.getCode().equalsIgnoreCase(captcha) && captchaEntity.getExpireTime().getTime() >= System.currentTimeMillis()){
            return true;
        }

        return false;
    }
}
